@@ -22,6 +22,7 @@
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
@@ -29,9 +30,13 @@
 #include <sys/types.h>
 #include <sys/time.h>
 
+#include "gnome.h"
 #include "panel-applet.h"
+#include "panel-applet-gconf.h"
 #include "gpmudmon.h"
 #include "gtkgauge.h"
+#include <gtkdatabox.h>
+#include <gtkdatabox_marker.h>
 
 GtkWidget *counter;
 GtkTooltips *tooltip;
@@ -39,6 +44,8 @@ GtkTooltips *tooltip;
 FILE *logfile;
 #endif
 
+extern void update_graph(PmudData *pd);
+extern void charge_cb(BonoboUIComponent *uic, gpointer data, const gchar *verbname);
 extern void prefs_cb(BonoboUIComponent *uic, gpointer data, const gchar *verbname);
 #ifdef PMUD
 static void sleep_cb(BonoboUIComponent *uic, gpointer data, const gchar *verbname);
@@ -52,8 +59,9 @@ static PMUD_STATUS * get_pmud_status(PmudData *pd);
 
 static const BonoboUIVerb GPmudMon_Applet_menu_verbs [] = {
 	        BONOBO_UI_VERB ("GPmudMon_Properties", prefs_cb),
-	        BONOBO_UI_VERB ("GPmudMon_Help",       help_cb),
-	        BONOBO_UI_VERB ("GPmudMon_About",      about_cb),
+	        BONOBO_UI_VERB ("GPmudMon_Graph", charge_cb),
+	        BONOBO_UI_VERB ("GPmudMon_Help", help_cb),
+	        BONOBO_UI_VERB ("GPmudMon_About", about_cb),
 	        BONOBO_UI_VERB_END
 };
 
@@ -118,17 +126,18 @@ about_cb(BonoboUIComponent *uic, gpointer d, const gchar *verbname)
 	authors[1] = "http://sourceforge.net/projects/gpmudmon";
 	authors[2] = NULL;
 
-	pixbuf = gpmudmon_load_pixbuf( "gpmudmon-applet/gpmudmon-logo.png");
+	pixbuf = gdk_pixbuf_new_from_file_at_size (ICONDIR"/gpmudmon-applet/gpmudmon-applet.svg",
+													 48, 48, NULL); //gpmudmon_load_pixbuf( "gpmudmon-applet/gpmudmon-logo.png");
 
-	about = GTK_WIDGET(GTK_DIALOG(gnome_about_new (
+	about = gnome_about_new (
 		"GNOME Pmu Monitor",
 		VERSION,
-		"(C) 2000-2004 Uwe Steinmann",
+		"(C) 2000-2007 Uwe Steinmann",
 		_("This applet displays the battery status as provided by the power mananagement unit on apple computers."),
 		authors,
 		documenters,
 		NULL,
-		pixbuf)));
+		pixbuf);
 
 	if(pixbuf)
 		gdk_pixbuf_unref (pixbuf);
@@ -194,6 +203,134 @@ void change_orient_cb(PanelApplet *applet, PanelAppletOrient o, PmudData *data)
 }
 /* }}} */
 
+/* graph_timer() {{{
+ *
+ */
+int
+graph_timer(gpointer data)
+{
+	PmudData *pd = (PmudData *) data;
+	BAT_STATUS *batleft, *batright;
+
+	pd->pmud_status = get_pmud_status(pd);
+	if(pd->pmud_status == NULL) {
+		return 1;
+	}
+	if(pd->valid_data == 0) {
+		batright = pd->pmud_status->bat;
+		batleft = batright++;
+
+		if(batleft->present) {
+			if(pd->sample_only == 0 || batleft->current != 0.0) {
+				memcpy(pd->graph_data_left->current+1, pd->graph_data_left->current, (pd->numsamples-1) * sizeof(gfloat));
+				memcpy(pd->graph_data_left->voltage+1, pd->graph_data_left->voltage, (pd->numsamples-1) * sizeof(gfloat));
+				pd->graph_data_left->current[0] = batleft->current;
+				pd->graph_data_left->voltage[0] = batleft->voltage*100.0;
+				int index = (int) (pd->markerx_left[0] + 0.5);
+				pd->markery_left[0] = pd->graph_data_left->current[index];
+				pd->markery_left[1] = pd->graph_data_left->voltage[index];
+
+				if(pd->databox_left) {
+					gchar *text;
+					guint position;
+					GtkDataboxCoord coord;
+					GtkDataboxValue value;
+					value.x = pd->markerx_left[0];
+					value.y = 0;
+					coord = gtk_databox_coord_from_value(GTK_DATABOX(pd->databox_left), value);
+					text = g_strdup_printf ("%.3f A", pd->graph_data_left->current[index]/1000.0);
+					if(coord.x > 475)
+						position = GTK_DATABOX_TEXT_NW;
+					else if(coord.x < 25)
+						position = GTK_DATABOX_TEXT_NE;
+					else
+						position = GTK_DATABOX_TEXT_N;
+					gtk_databox_marker_set_label (GTK_DATABOX_MARKER(pd->marker_left), 0,
+																				position, text, TRUE);
+					g_free((gpointer) text);
+					text = g_strdup_printf ("%.3f V", pd->graph_data_left->voltage[index]/100.0);
+					if(coord.x > 475)
+						position = GTK_DATABOX_TEXT_SW;
+					else if(coord.x < 25)
+						position = GTK_DATABOX_TEXT_SE;
+					else
+						position = GTK_DATABOX_TEXT_S;
+					gtk_databox_marker_set_label (GTK_DATABOX_MARKER(pd->marker_left), 1,
+																				position, text, TRUE);
+					g_free((gpointer) text);
+
+					gtk_databox_redraw(pd->databox_left);
+
+					int i;
+					float charge = 0.0;
+					for(i=0; i<pd->numsamples; i++) {
+						if(pd->graph_data_left->current[i] != 0)
+							charge += pd->graph_data_left->current[i]/3600.0 * (pd->graphupdatetime/1000.0);
+					}
+					text=g_strdup_printf(_("%.0f mAh"), charge);
+					gtk_entry_set_text(GTK_ENTRY(pd->charge_left), text);
+					g_free((gpointer) text);
+				}
+			}
+		}
+		if(batright->present) {
+			if(pd->sample_only == 0 || batright->current != 0.0) {
+				memcpy(pd->graph_data_right->current+1, pd->graph_data_right->current, (pd->numsamples-1) * sizeof(gfloat));
+				memcpy(pd->graph_data_right->voltage+1, pd->graph_data_right->voltage, (pd->numsamples-1) * sizeof(gfloat));
+				pd->graph_data_right->current[0] = batright->current;
+				pd->graph_data_right->voltage[0] = batright->voltage*100.0;
+				int index = (int) (pd->markerx_right[0] + 0.5);
+				pd->markery_right[0] = pd->graph_data_right->current[index];
+				pd->markery_right[1] = pd->graph_data_right->voltage[index];
+
+				if(pd->databox_right) {
+					gchar *text;
+					guint position;
+					GtkDataboxCoord coord;
+					GtkDataboxValue value;
+					value.x = pd->markerx_right[0];
+					value.y = 0;
+					coord = gtk_databox_coord_from_value(GTK_DATABOX(pd->databox_right), value);
+					text = g_strdup_printf ("%.3f A", pd->graph_data_right->current[index]/1000.0);
+					if(coord.x > 475)
+						position = GTK_DATABOX_TEXT_NW;
+					else if(coord.x < 25)
+						position = GTK_DATABOX_TEXT_NE;
+					else
+						position = GTK_DATABOX_TEXT_N;
+					gtk_databox_marker_set_label (GTK_DATABOX_MARKER(pd->marker_right), 0,
+																				position, text, TRUE);
+					g_free((gpointer) text);
+					text = g_strdup_printf ("%.3f V", pd->graph_data_right->voltage[index]/100.0);
+					if(coord.x > 475)
+						position = GTK_DATABOX_TEXT_SW;
+					else if(coord.x < 25)
+						position = GTK_DATABOX_TEXT_SE;
+					else
+						position = GTK_DATABOX_TEXT_S;
+					gtk_databox_marker_set_label (GTK_DATABOX_MARKER(pd->marker_right), 1,
+																				position, text, TRUE);
+					g_free((gpointer) text);
+
+					gtk_databox_redraw(pd->databox_right);
+
+					int i;
+					float charge = 0.0;
+					for(i=0; i<pd->numsamples; i++) {
+						if(pd->graph_data_right->current[i] != 0)
+							charge += pd->graph_data_right->current[i]/3600.0 * (pd->graphupdatetime/1000.0);
+					}
+					text=g_strdup_printf(_("%.0f mAh"), charge);
+					gtk_entry_set_text(GTK_ENTRY(pd->charge_right), text);
+					g_free((gpointer) text);
+				}
+			}
+		}
+	}
+	return 1;
+}
+/* }}} */
+
 /* count_timer() {{{
  *
  */
@@ -256,7 +393,7 @@ count_timer(gpointer data)
 		if(batright->present) {
 			if(batleft->present) {
 				tiptext = realloc(tiptext, strlen(tiptext)+2+1);
-				strcat(tiptext, "\n");
+//				strcat(tiptext, "\n");
 			}
 			gtk_gauge_set_value(GTK_GAUGE(pd->gauge2), batright->current);
 			gtk_widget_show(pd->bar2);
@@ -470,7 +607,7 @@ void get_value_int(char *buf, int *ret) {
 static PMUD_STATUS *
 get_pmud_status(PmudData *pd)
 {
-	char buf[IM_BUFSIZE], buf2[IM_BUFSIZE], *ptr1;
+	char buf[IM_BUFSIZE];
 	FILE *pmuinfo, *pmubat[2];
 	int i, numbats, acpower;
 	PMUD_STATUS *status = NULL;
@@ -488,7 +625,6 @@ get_pmud_status(PmudData *pd)
 	}
 	numbats = 0;
 	while(!feof(pmuinfo)) {
-		char *str;
 		fgets(buf, IM_BUFSIZE-1, pmuinfo);
 		if(!strncmp(buf, "Battery count", 13)) {
 			get_value_int(buf, &numbats);
@@ -502,7 +638,6 @@ get_pmud_status(PmudData *pd)
 
 	write_log("Found %d batteries\n", numbats);
 	if(numbats) {
-		int error;
 		char flags[20];
 
 		for(i=0; i<numbats; i++) {
@@ -512,7 +647,6 @@ get_pmud_status(PmudData *pd)
 			if(pmubat[i]) {
 				write_log("Successfully opened file '%s'\n", buf);
 				while(!feof(pmubat[i])) {
-					char *str;
 					fgets(buf, IM_BUFSIZE-1, pmubat[i]);
 					if(!strncmp(buf, "flags", 5)) {
 						get_value_string(buf, flags);
@@ -617,11 +751,14 @@ static GtkWidget
 void
 save_config(PmudData *pd) {
 	panel_applet_gconf_set_int(PANEL_APPLET(pd->app), "updatetime", pd->updatetime, NULL);
+	panel_applet_gconf_set_int(PANEL_APPLET(pd->app), "graphupdatetime", pd->graphupdatetime, NULL);
+	panel_applet_gconf_set_int(PANEL_APPLET(pd->app), "numsamples", pd->numsamples, NULL);
 	panel_applet_gconf_set_int(PANEL_APPLET(pd->app), "width", pd->width, NULL);
 	panel_applet_gconf_set_int(PANEL_APPLET(pd->app), "height", pd->height, NULL);
 	panel_applet_gconf_set_int(PANEL_APPLET(pd->app), "port", pd->port, NULL);
 	panel_applet_gconf_set_string(PANEL_APPLET(pd->app), "name", pd->servername, NULL);
-	panel_applet_gconf_set_int(PANEL_APPLET(pd->app), "showcurrent", pd->show_current, NULL);
+	panel_applet_gconf_set_bool(PANEL_APPLET(pd->app), "showcurrent", pd->show_current, NULL);
+	panel_applet_gconf_set_bool(PANEL_APPLET(pd->app), "sampleonly", pd->sample_only, NULL);
 }
 /* }}} */
 
@@ -645,11 +782,24 @@ applet_pixel_size_changed_cb (GtkWidget *applet, int size, PmudData *pd)
  */
 void
 load_config(PmudData *pd) {
-	GError *error;
+	int itmp;
+	GError *error = NULL;
 
-	pd->updatetime = panel_applet_gconf_get_int(PANEL_APPLET(pd->app), "updatetime", NULL);
-	if(pd->updatetime < 1000)
-		pd->updatetime = 1000;
+	itmp = panel_applet_gconf_get_int(PANEL_APPLET(pd->app), "updatetime", &error);
+	if(!error) {
+		pd->updatetime = MAX(itmp, 1000);
+	}
+	error = NULL;
+	itmp = panel_applet_gconf_get_int(PANEL_APPLET(pd->app), "graphupdatetime", &error);
+	if(!error) {
+		pd->graphupdatetime = MAX(itmp, 1000);
+	}
+	error = NULL;
+	itmp = panel_applet_gconf_get_int(PANEL_APPLET(pd->app), "numsamples", &error);
+	if(!error) {
+		pd->numsamples = MIN(itmp, 1000);
+	}
+	error = NULL;
 	pd->width = panel_applet_gconf_get_int(PANEL_APPLET(pd->app), "width", NULL);
 	pd->height = panel_applet_gconf_get_int(PANEL_APPLET(pd->app), "height", NULL);
 	pd->port = panel_applet_gconf_get_int(PANEL_APPLET(pd->app), "port", NULL);
@@ -658,7 +808,8 @@ load_config(PmudData *pd) {
 	pd->servername = (char *) panel_applet_gconf_get_string(PANEL_APPLET(pd->app), "name", NULL);
 	if(pd->servername == NULL)
 		pd->servername = (char *) strdup("localhost");
-	pd->show_current = panel_applet_gconf_get_int(PANEL_APPLET(pd->app), "showcurrent", NULL);
+	pd->show_current = panel_applet_gconf_get_bool(PANEL_APPLET(pd->app), "showcurrent", NULL);
+	pd->sample_only = panel_applet_gconf_get_bool(PANEL_APPLET(pd->app), "sampleonly", NULL);
 }
 /* }}} */
 
@@ -669,13 +820,22 @@ static gboolean
 gpmudmon_applet_fill (PanelApplet *applet)
 {
 	PmudData *pd;
+	int i;
 
+	panel_applet_add_preferences (PANEL_APPLET (applet), "/schemas/apps/gpmudmon-applet/prefs", NULL);
 	pd = g_new0(PmudData, 1);
 	pd->app = applet;
 	pd->port = 879;
 	pd->servername = (char *) strdup("localhost");
 	pd->updatetime = 10000;
+	pd->graphupdatetime = 10000;
+	pd->numsamples = GRAPH_DATA_SAMPLES;
 	pd->show_current = 1;
+	pd->sample_only = 1;
+	pd->ch = NULL;
+	pd->pb = NULL;
+	pd->charge_left = NULL;
+	pd->charge_right = NULL;
 
 	load_config(pd);
 	pd->globalbox = create_monitor(pd);
@@ -686,6 +846,37 @@ gpmudmon_applet_fill (PanelApplet *applet)
 	pd->pmudinfo = open_pmudinfo(pd);
 #endif
 	pd->pmud_status = NULL;
+
+	pd->graph_data_left = (GRAPH_DATA *) malloc(sizeof(GRAPH_DATA));
+	pd->graph_data_left->current = (float *) malloc(pd->numsamples * sizeof(float));
+	pd->graph_data_left->voltage = (float *) malloc(pd->numsamples * sizeof(float));
+	pd->graph_data_left->xindex = (float *) malloc(pd->numsamples * sizeof(float));
+	memset(pd->graph_data_left->current, 0, pd->numsamples * sizeof(float));
+	memset(pd->graph_data_left->voltage, 0, pd->numsamples * sizeof(float));
+	for(i=0; i< pd->numsamples; i++)
+		pd->graph_data_left->xindex[i] = (gfloat) i;
+	pd->markerx_left = g_new0 (gfloat, 2);
+	pd->markery_left = g_new0 (gfloat, 2);
+	pd->markerx_left[0] = 0;
+	pd->markery_left[0] = 0;
+	pd->markerx_left[1] = 0;
+	pd->markery_left[1] = 0;
+
+	pd->graph_data_right = (GRAPH_DATA *) malloc(sizeof(GRAPH_DATA));
+	pd->graph_data_right->current = (float *) malloc(pd->numsamples * sizeof(float));
+	pd->graph_data_right->voltage = (float *) malloc(pd->numsamples * sizeof(float));
+	pd->graph_data_right->xindex = (float *) malloc(pd->numsamples * sizeof(float));
+	memset(pd->graph_data_right->current, 0, pd->numsamples * sizeof(float));
+	memset(pd->graph_data_right->voltage, 0, pd->numsamples * sizeof(float));
+	for(i=0; i< pd->numsamples; i++)
+		pd->graph_data_right->xindex[i] = (gfloat) i;
+	pd->markerx_right = g_new0 (gfloat, 2);
+	pd->markery_right = g_new0 (gfloat, 2);
+	pd->markerx_right[0] = 0;
+	pd->markery_right[0] = 0;
+	pd->markerx_right[1] = 0;
+	pd->markery_right[1] = 0;
+
 	gtk_container_add(GTK_CONTAINER(pd->app), pd->globalbox);
 
   panel_applet_setup_menu_from_file (PANEL_APPLET(applet),
@@ -708,6 +899,8 @@ gpmudmon_applet_fill (PanelApplet *applet)
 #endif
 	count_timer(pd);
 	pd->timeout = gtk_timeout_add(pd->updatetime, (GtkFunction)count_timer, pd);
+	graph_timer(pd);
+	pd->graphtimeout = gtk_timeout_add(pd->graphupdatetime, (GtkFunction)graph_timer, pd);
 	return(TRUE);
 }
 /* }}} */
