@@ -1,7 +1,6 @@
-/* ISDN Monitor panel applet
- * Insprired by kimon by Markus Dahlweid dahlweid@tzi.org 
+/* pmud Monitor panel applet
  *
- * By Uwe Steinmann <Uwe.Steinmann@fernuni-hagen.de>
+ * By Uwe Steinmann <uwe@steinmann.cx>
  *
  * Enjoy.
  *
@@ -28,6 +27,13 @@
 #include <gtk/gtk.h>
 #include <panel-applet.h>
 
+#define GRAPH_DATA_SAMPLES 2000
+typedef struct _GRAPH_DATA {
+	gfloat *xindex;    /* used for x coordinate */
+	gfloat *current;
+	gfloat *voltage;
+} GRAPH_DATA;
+
 typedef struct _BAT_STATUS {
 	int present;
 	int ac_on;
@@ -47,6 +53,14 @@ typedef struct _PmudData {
 	PanelApplet *app;
 	GtkWidget *globalbox;
 	GtkWidget *pb;        // Property box
+	GtkWidget *ch;        // charge box
+	GtkWidget *databox_left, *databox_right;
+	GtkWidget *marker_left, *marker_right;
+	gfloat *markerx_left, *markerx_right;
+	gfloat *markery_left, *markery_right;
+	GtkWidget *xcoord_left, *ycoord_left;
+	GtkWidget *xcoord_right, *ycoord_right;
+	GtkWidget *charge_left, *charge_right;
 	GtkWidget *bar1, *bar2;
 	GtkWidget *gauge1, *gauge2;
 	GtkWidget *gaugeframe1, *gaugeframe2;
@@ -59,13 +73,19 @@ typedef struct _PmudData {
 	int height, width;
 	int updatetime;
 	int timeout;
+	int graphupdatetime;
+	int graphtimeout;
+	int numsamples;
 	int port;
 	char *servername;
-	int show_current;
+	gboolean show_current;
+	gboolean sample_only;
 
 	PMUD_STATUS *pmud_status;
 	PMUD_STATUS *old_pmud_status;
 	int valid_data;      /* 0 if current pmud_status is valid */
+
+	GRAPH_DATA *graph_data_left, *graph_data_right;
 #ifdef PMUD
 	int pmudinfo;
 #else
