@@ -424,9 +424,9 @@ gtk_gauge_realize (GtkWidget *widget)
 {
 	GtkGauge       *gauge;
 	GdkColormap  *cmap;
-	GdkWindowAttr attributes;
+/*	GdkWindowAttr attributes;
 	gint attributes_mask;
-
+*/
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (GTK_IS_GAUGE (widget));
 
@@ -473,8 +473,6 @@ gtk_gauge_realize (GtkWidget *widget)
 static void
 gtk_gauge_update(GtkGauge *gauge)
 {
-	gfloat new_value;
-		  
 	g_return_if_fail (gauge != NULL);
 	g_return_if_fail (GTK_IS_GAUGE (gauge));
 
@@ -487,7 +485,7 @@ static gboolean
 gtk_gauge_expose(GtkWidget *widget, GdkEventExpose *event) {
 	GtkGauge   *gauge;
 	GdkColor *win_bg;
-	gint     yradius, xradius, value, strwth;
+	gint     yradius, value, strwth;
 	float    angle;
 	char buffer[10];
 
@@ -516,7 +514,7 @@ gtk_gauge_expose(GtkWidget *widget, GdkEventExpose *event) {
 
 //fprintf(stderr, "gtkgauge: x = %d, y = %d, height = %d, width = %d\n", x, y, widget->allocation.height, widget->allocation.width);
 			if(widget->allocation.height < 5)
-				return;
+				return FALSE;
 
 			yradius = widget->allocation.height-2*BORDER;	
 
