@@ -32,6 +32,7 @@
 
 extern int open_pmudinfo();
 extern int count_timer(gpointer data);
+extern int graph_timer(gpointer data);
 extern void save_config(PmudData *pd);
 
 static void apply_cb(GtkObject *pb, gint page, gpointer data)
@@ -43,6 +44,8 @@ static void apply_cb(GtkObject *pb, gint page, gpointer data)
 //	GtkWidget *width_s = gtk_object_get_data(GTK_OBJECT(pb),"width");
 //	GtkWidget *height_s = gtk_object_get_data(GTK_OBJECT(pb),"height");
 	GtkWidget *update_s = gtk_object_get_data(GTK_OBJECT(pb), "update");
+	GtkWidget *update_graph_s = gtk_object_get_data(GTK_OBJECT(pb), "graph_update");
+	GtkWidget *num_samples = gtk_object_get_data(GTK_OBJECT(pb), "num_samples");
 
 	/* Do not make the changes on page basis, but for all pages */
 	if(page != -1)
@@ -52,10 +55,46 @@ static void apply_cb(GtkObject *pb, gint page, gpointer data)
 //	sscanf(buf, "%d", &(pd->height));
 //	buf = gtk_entry_get_text(GTK_ENTRY(width_s));
 //	sscanf(buf, "%d", &(pd->width));
-	buf = gtk_entry_get_text(GTK_ENTRY(update_s));
-	sscanf(buf, "%d", &(pd->updatetime));
-	gtk_timeout_remove(pd->timeout);
-	pd->timeout = gtk_timeout_add(pd->updatetime, (GtkFunction)count_timer, pd);
+	if(update_s) {
+		buf = gtk_entry_get_text(GTK_ENTRY(update_s));
+		sscanf(buf, "%d", &(pd->updatetime));
+		gtk_timeout_remove(pd->timeout);
+		pd->timeout = gtk_timeout_add(pd->updatetime, (GtkFunction)count_timer, pd);
+	}
+
+	if(update_graph_s) {
+		buf = gtk_entry_get_text(GTK_ENTRY(update_graph_s));
+		sscanf(buf, "%d", &(pd->graphupdatetime));
+		gtk_timeout_remove(pd->graphtimeout);
+		pd->graphtimeout = gtk_timeout_add(pd->graphupdatetime, (GtkFunction)graph_timer, pd);
+	}
+
+	if(num_samples) {
+		int itmp, i;
+		buf = gtk_entry_get_text(GTK_ENTRY(num_samples));
+		sscanf(buf, "%d", &itmp);
+		if(itmp != pd->numsamples) {
+			pd->numsamples = itmp;
+
+			pd->graph_data_left->current = (float *) realloc(pd->graph_data_left->current, pd->numsamples * sizeof(float));
+			pd->graph_data_left->voltage = (float *) realloc(pd->graph_data_left->voltage, pd->numsamples * sizeof(float));
+			pd->graph_data_left->xindex = (float *) realloc(pd->graph_data_left->xindex, pd->numsamples * sizeof(float));
+			for(i=0; i<pd->numsamples; i++)
+				pd->graph_data_left->xindex[i] = (gfloat) i;
+			create_graph_left(pd);
+			set_graph_scale_left(pd);
+			gtk_databox_redraw(pd->databox_left);
+
+			pd->graph_data_right->current = (float *) realloc(pd->graph_data_right->current, pd->numsamples * sizeof(float));
+			pd->graph_data_right->voltage = (float *) realloc(pd->graph_data_right->voltage, pd->numsamples * sizeof(float));
+			pd->graph_data_right->xindex = (float *) realloc(pd->graph_data_right->xindex, pd->numsamples * sizeof(float));
+			for(i=0; i<pd->numsamples; i++)
+				pd->graph_data_right->xindex[i] = (gfloat) i;
+			create_graph_right(pd);
+			set_graph_scale_right(pd);
+			gtk_databox_redraw(pd->databox_right);
+		}
+	}
 
 #ifdef PMUD
 	/*** servername ***/
@@ -82,6 +121,10 @@ static void apply_cb(GtkObject *pb, gint page, gpointer data)
 		gtk_widget_hide(pd->gaugeframe2);
 	}
 
+	/**** Sample only of current != 0 ****/
+	widget_s = gtk_object_get_data(GTK_OBJECT(pb), "sampleonly");
+	pd->sample_only = GTK_TOGGLE_BUTTON(widget_s)->active ? 1 : 0;
+
 	save_config(pd);
 #ifdef PMUD
 	pd->pmudinfo = open_pmudinfo(pd);
@@ -106,22 +149,31 @@ changed_cb(gpointer d)
 void
 prefs_cb(BonoboUIComponent *uic, gpointer p, const gchar *verbname)
 {
-	GtkWidget *frame, *hbox, *tbox, *label, *spin_label, *cadre, *entry, *checkbox;
+	GtkWidget *frame, *hbox, *tbox, *label, *spin_label, *cadre, *checkbox;
+#ifdef PMUD
+	GtkWidget *entry;
+#endif
 	GtkAdjustment *adj;
 	PmudData *pd = (PmudData *) p;
 
+	if(pd->pb) {
+		gtk_window_set_screen (GTK_WINDOW (pd->pb),
+				       gtk_widget_get_screen (GTK_WIDGET (pd->app)));
+		gtk_window_present (GTK_WINDOW (pd->pb));
+		return;
+	}
+
 	pd->pb = gnome_property_box_new();
 	gtk_window_set_title(GTK_WINDOW(pd->pb), _("gpmudmon Preferences"));
 	
 	frame=gtk_vbox_new(FALSE,0);
-
 	gnome_property_box_append_page(GNOME_PROPERTY_BOX(pd->pb), frame,
                        gtk_label_new(_("General")));
 	
 	hbox=gtk_hbox_new(FALSE,5);
 	gtk_box_pack_start(GTK_BOX(frame),hbox,FALSE,FALSE,5);
 	gtk_widget_show(hbox);
-	label=gtk_label_new(_("Refresh rate (in milli seconds)"));
+	label=gtk_label_new(_("Refresh rate (in ms)"));
 	gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,5);
 	gtk_widget_show(label);
 	adj=(GtkAdjustment *)gtk_adjustment_new(pd->updatetime,100,200000,1000,100,10);
@@ -133,13 +185,12 @@ prefs_cb(BonoboUIComponent *uic, gpointer p, const gchar *verbname)
 	gtk_signal_connect_object(GTK_OBJECT(adj),"value_changed",GTK_SIGNAL_FUNC(changed_cb),(gpointer)pd);
 	gtk_signal_connect_object(GTK_OBJECT(spin_label),"changed",GTK_SIGNAL_FUNC(changed_cb),(gpointer)pd);
 
-	cadre=gtk_frame_new(_("Current"));
-	gtk_widget_show(cadre);
-	tbox=gtk_vbox_new(FALSE,5);
-	gtk_widget_show(tbox);
+//	cadre=gtk_frame_new(_("Current"));
+//	tbox=gtk_vbox_new(FALSE,5);
 
 	hbox=gtk_hbox_new(FALSE,5);
-	gtk_box_pack_start(GTK_BOX(tbox),hbox,FALSE,FALSE, 2);
+	gtk_box_pack_start(GTK_BOX(frame),hbox,FALSE,FALSE, 2);
+//	gtk_box_pack_start(GTK_BOX(tbox),hbox,FALSE,FALSE, 2);
 	gtk_widget_show(hbox);
 	label=gtk_label_new(_("Current gauge"));
 	gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,5);
@@ -152,8 +203,8 @@ prefs_cb(BonoboUIComponent *uic, gpointer p, const gchar *verbname)
 	gtk_signal_connect(GTK_OBJECT(checkbox), "toggled",
                    	GTK_SIGNAL_FUNC(toggled_cb), (gpointer) pd);
 
-	gtk_container_add(GTK_CONTAINER(cadre),tbox);
-	gtk_box_pack_start(GTK_BOX(frame),cadre,FALSE,FALSE,15);
+//	gtk_container_add(GTK_CONTAINER(cadre),tbox);
+//	gtk_box_pack_start(GTK_BOX(frame),cadre,FALSE,FALSE,15);
 
 #ifdef PMUD
 	cadre=gtk_frame_new(_("Server"));
@@ -194,6 +245,54 @@ prefs_cb(BonoboUIComponent *uic, gpointer p, const gchar *verbname)
 	gtk_box_pack_start(GTK_BOX(frame),cadre,FALSE,FALSE,15);
 #endif
 
+	frame=gtk_vbox_new(FALSE,0);
+	gnome_property_box_append_page(GNOME_PROPERTY_BOX(pd->pb), frame,
+                       gtk_label_new(_("Graph")));
+
+	hbox=gtk_hbox_new(FALSE,5);
+	gtk_box_pack_start(GTK_BOX(frame),hbox,FALSE,FALSE,5);
+	gtk_widget_show(hbox);
+	label=gtk_label_new(_("Number of samples"));
+	gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,5);
+	gtk_widget_show(label);
+	adj=(GtkAdjustment *)gtk_adjustment_new(pd->numsamples,100,200000,1000,100,10);
+	spin_label=gtk_spin_button_new(adj,1,0);
+	gtk_widget_set_usize(spin_label, 70,-1);
+	gtk_object_set_data(GTK_OBJECT(pd->pb), "num_samples", spin_label);
+	gtk_box_pack_start(GTK_BOX(hbox),spin_label,FALSE,FALSE,5);
+	gtk_widget_show(spin_label);
+	gtk_signal_connect_object(GTK_OBJECT(adj),"value_changed",GTK_SIGNAL_FUNC(changed_cb),(gpointer)pd);
+	gtk_signal_connect_object(GTK_OBJECT(spin_label),"changed",GTK_SIGNAL_FUNC(changed_cb),(gpointer)pd);
+
+	hbox=gtk_hbox_new(FALSE,5);
+	gtk_box_pack_start(GTK_BOX(frame),hbox,FALSE,FALSE,5);
+	gtk_widget_show(hbox);
+	label=gtk_label_new(_("Sample time (in ms)"));
+	gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,5);
+	gtk_widget_show(label);
+	adj=(GtkAdjustment *)gtk_adjustment_new(pd->graphupdatetime,100,200000,1000,100,10);
+	spin_label=gtk_spin_button_new(adj,1,0);
+	gtk_widget_set_usize(spin_label, 70,-1);
+	gtk_object_set_data(GTK_OBJECT(pd->pb), "graph_update", spin_label);
+	gtk_box_pack_start(GTK_BOX(hbox),spin_label,FALSE,FALSE,5);
+	gtk_widget_show(spin_label);
+	gtk_signal_connect_object(GTK_OBJECT(adj),"value_changed",GTK_SIGNAL_FUNC(changed_cb),(gpointer)pd);
+	gtk_signal_connect_object(GTK_OBJECT(spin_label),"changed",GTK_SIGNAL_FUNC(changed_cb),(gpointer)pd);
+
+	hbox=gtk_hbox_new(FALSE,5);
+	gtk_box_pack_start(GTK_BOX(frame),hbox,FALSE,FALSE, 2);
+	gtk_widget_show(hbox);
+	label=gtk_label_new(_("Sample only if current != 0"));
+	gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,5);
+	gtk_widget_show(label);
+	checkbox = gtk_check_button_new_with_label (_("On/Off"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox), pd->sample_only);
+	gtk_object_set_data(GTK_OBJECT(pd->pb), "sampleonly", checkbox);
+	gtk_box_pack_start(GTK_BOX(hbox),checkbox,FALSE,FALSE,5);
+	gtk_widget_show(checkbox);
+	gtk_signal_connect(GTK_OBJECT(checkbox), "toggled",
+                   	GTK_SIGNAL_FUNC(toggled_cb), (gpointer) pd);
+
 	gtk_widget_show(frame);
 
 	gtk_signal_connect(GTK_OBJECT(pd->pb), "apply",
